import direct.directbase.DirectStart
from panda3d.core import WindowProperties
from panda3d.core import Filename,Shader
from panda3d.core import AmbientLight,PointLight, DirectionalLight, Spotlight, PerspectiveLens
from panda3d.core import Texture, TextureStage,TexGenAttrib, VBase4 
from panda3d.core import TextNode, LODNode
from panda3d.core import Point3,Vec3,Vec4
from panda3d.core import CollisionTraverser, CollisionHandlerPusher
from panda3d.core import CollisionNode, CollisionSphere
from direct.task.Task import Task
from direct.actor.Actor import Actor
from direct.gui.OnscreenText import OnscreenText
from direct.showbase.DirectObject import DirectObject
from direct.filter.CommonFilters import *
import sys,os, math
from random import random, randrange, randint
 
 
#helpers
targeting_area = (50/800* base.win.getXSize(), 50/600* base.win.getXSize())
 
# Function to put title on the screen.
def addTitle(text):
    return OnscreenText(text=text, style=1, fg=(1,1,1,1),
                        pos=(1.3,-0.95), align=TextNode.ARight, scale = .07)
def addInstructions(pos, msg):
    return OnscreenText(text=msg, style=1, fg=(1,1,1,1),
                        pos=(-1.3, pos), align=TextNode.ALeft, scale = .05)

class SkySphere(DirectObject): 
    def __init__(self):       
        self.sphere = loader.loadModel("InvertedSphere.egg") 
        self.sphere.setTexGen(TextureStage.getDefault(), TexGenAttrib.MWorldPosition) 
        self.sphere.setTexProjector(TextureStage.getDefault(), render, self.sphere) 
        self.sphere.setTexPos(TextureStage.getDefault(), 0, 0, 0) 
        self.sphere.setTexScale(TextureStage.getDefault(), .5) 
        tex = loader.loadCubeMap("tex/sky/sky_#.png", okMissing = True) 
        self.sphere.setTexture(tex) 
        self.sphere.setLightOff() 
        self.sphere.setScale(1000) 
        self.sphere.reparentTo(render) 
    def model(self):
        return self.sphere;
                        
class Asteroid:
    def __init__(self, _scale = 1, _position = (0,0,0), _models = None):
        self.model_vnear = None
        self.model_near = None
        self.model_middle = None
        self.model_far = None
        """
        if not _models:
            self.model_vnear = loader.loadModel('aster_verynear.egg')
            self.model_near = loader.loadModel('aster_near.egg')
            self.model_middle = loader.loadModel('aster_middle.egg')
            self.model_far = loader.loadModel('aster_far.egg')
          
        else:
            self.model_vnear = loader.loadModel(_models[0])
            self.model_near = loader.loadModel(_models[1])
            self.model_middle = loader.loadModel(_models[2])
            self.model_far = loader.loadModel(_models[3])
            
        lod = LODNode('Asteroid')
        lod_np = NodePath(lod)
        lod_np.reparentTo(render)

        lod.addSwitch(100.0, 0.0)
        self.model_vnear.reparentTo(lod_np)        
        lod.addSwitch(300.0, 100.0)
        self.model_near.reparentTo(lod_np)            
        lod.addSwitch(2500.0, 300.0)
        self.model_middle.reparentTo(lod_np)     
        lod.addSwitch(15000.0, 2500.0)
        self.model_far.reparentTo(lod_np)    
        
        self.model = lod_np
        """
        self.model = loader.loadModel('aster_near.egg').find('**/+GeomNode')
        self.model.setScale(_scale*(1+randrange(-10,10)/50.0), \
                             _scale*(1+randrange(-10,10)/50.0), \
                             _scale*(1+randrange(-10,10)/50.0))
        self.model.setPos(_position)
        self.model.setHpr(randrange(0,360),randrange(0,360),randrange(0,360))
        self.model.reparentTo(render)
        #self.model.setShaderAuto()
    def node(self):
        return self.model

class Ship:
    def __init__(self):
        self.model = loader.loadModel('ship.egg')
        self.model.setScale(11,11,11)
        
    def node(self):
        return self.model
    def to_player(self):
        base.camera.setPos(-10,0,10)
        base.camera.lookAt(self.model)
        self.model.reparentTo(render)
        base.camera.reparentTo(self.model)

    def to_mob(self):
        self.model.reparentTo(render)        
        
class Main(DirectObject):
    def __init__(self):
        print 'Init stage 1'
        self.field = render.attachNewNode('a_field')
        # Initialize the collision traverser.
        base.cTrav = CollisionTraverser()
         
        # Initialize the Pusher collision handler.
        self.pusher = CollisionHandlerPusher()
        # self.title1 = addTitle('Direction')
        
        # self.inst1 = addInstructions(0.95, "Player")
        # self.inst2 = addInstructions(0.90, "Direction")
        # self.inst3 = addInstructions(0.85, "RelPlayer")
        # self.inst4 = addInstructions(0.80, "--")
        # self.inst5 = addInstructions(0.75, "--")
        

        # Check video card capabilities.

        if (base.win.getGsg().getSupportsBasicShaders() == 0):
            addTitle("Bump Mapping: Video driver reports that shaders are not supported.")
            return
        # Make the mouse invisible, turn off normal mouse controls
        #base.disableMouse()
        props = WindowProperties()
        #props.setCursorHidden(True)
        base.win.requestProperties(props)

        #VARIABLES
        self.mousex = 0
        self.mousey = 0
        self.last = 0
        self.mousebtn = [0,0,0]
        self.position = Vec3(0,0,0)
        #self.forward = 0
        self.vector = Vec3(0,0,0)
        self.asteroids=[]
        
        print 'Init stage 2'
        taskMgr.add(self.controlCamera, "camera-task")
        self.accept("escape", sys.exit, [0])
        self.accept("mouse1", self.setMouseBtn, [0, 1])
        self.accept("mouse1-up", self.setMouseBtn, [0, 0])
        self.accept("mouse2", self.setMouseBtn, [1, 1])
        self.accept("mouse2-up", self.setMouseBtn, [1, 0])
        self.accept("mouse3", self.setMouseBtn, [2, 1])
        self.accept("mouse3-up", self.setMouseBtn, [2, 0])
        
        self.keyMap = {"w" : False,   
                     "a" : False,   
                     "s" : False,   
                     "d" : False,   
                     "q" : False,   
                     "e" : False,  
                     "shift" : False}   
        self.accept("w", self.setKey, ["w", True])   
        self.accept("a", self.setKey, ["a", True])   
        self.accept("s", self.setKey, ["s", True])   
        self.accept("d", self.setKey, ["d", True])   
        self.accept("q", self.setKey, ["q", True])   
        self.accept("e", self.setKey, ["e", True])          
        
        self.accept("shift", self.setKey, ["shift", True])   
        self.accept("w-up", self.setKey, ["w", False])   
        self.accept("a-up", self.setKey, ["a", False])   
        self.accept("s-up", self.setKey, ["s", False])   
        self.accept("d-up", self.setKey, ["d", False])  
        self.accept("q-up", self.setKey, ["q", False])   
        self.accept("e-up", self.setKey, ["e", False])           
        self.accept("shift-up", self.setKey, ["shift", False])   
        
        #World    
        sky = self.createSky()
        self.GenerateAsteroids()
        self.player = render.attachNewNode("Player")
        self.player.setHpr(0,0,0)
        # Create a collision node for this object.
        cNode = CollisionNode('CPlayer')
        # Attach a collision sphere solid to the collision node.
        cNode.addSolid(CollisionSphere(0, 0, 0,3))
        # Attach the collision node to the object's model.
        self.pcollid= self.player.attachNewNode(cNode)
        # Set the object's collision node to render as visible.
        self.pcollid.show()
        
        sky.model().reparentTo(self.player) 
        self.relplayer = self.player.attachNewNode("Relative Player")
        self.relplayer.setHpr(0,0,0)
        base.camera.setHpr(0,0,0)
        self.position = self.relplayer.getPos()
        base.camera.reparentTo(self.relplayer)
        #a =Ship()
        #a.to_mob()
        #self.player.to_player()
        #self.player.model.setPos(0,0,0)
        base.setBackgroundColor(0,0,0)    
        print 'Init stage 3'

        # dlight = DirectionalLight('dlight')
        # dlight.setColor(Vec4(0.05, 0.05, 0.05, 1))
        #dlight.setShadowCaster(True, 512, 512)
        # dlnp = render.attachNewNode(dlight)
        # dlnp.setHpr(0, -60, 0)
        #render.setLight(dlnp)

        spotlight = camera.attachNewNode( Spotlight( "spotlight" ) )
        spotlight.node().setColor( Vec4( 1, 1, 1, 1 ) )
        spotlight.node().setShadowCaster(True, 512, 512)
        spotlight.node().setLens( PerspectiveLens() )
        spotlight.node().getLens().setFov( 33, 33 )
        spotlight.node().setAttenuation( Vec3( 0.0, 0.01, 0.0 ) ) 
        spotlight.node().setExponent( 128.0 )
        render.setLight( spotlight )
        
        #FOG
        # colour = (0.1,0.1,0.1)
        # expfog = Fog("FOG")
        # expfog.setColor(*colour)
        # expfog.setExpDensity(0.0005)
        # render.setFog(expfog)
        #base.setBackgroundColor(*colour)

        
        print 'Init stage 4: fin'
        render.setShaderAuto()              


    def setKey(self,key,value):   
        self.keyMap[key] = value   

    def createSky(self):
        self.skysphere = SkySphere()
        self.skysphere.model().setBin('background', 1) 
        self.skysphere.model().setDepthWrite(0) 
        return self.skysphere


        
    def GenerateAsteroids(self):
        for x in range(-4,4):
            for y in range(-4,4):
                for z in range(-4,4):
                    scale=nx=ny=nz = 0
                    if randint(1,11) == 1:
                        scale = 150
                        nx = x*250 + randrange(-20,20,1) 
                        ny = y*250 + randrange(-20,20,1) 
                        nz = z*250 + randrange(-20,20,1) 
                    else:
                        scale = randrange(5,50,1)
                        nx = x*250 + randrange(-(80-scale),(80-scale),1) 
                        ny = y*250 + randrange(-(80-scale),(80-scale),1) 
                        nz = z*250 + randrange(-(80-scale),(80-scale),1) 
                    asteroid = Asteroid(scale, (nx,ny,nz))
                    asteroid.model.reparentTo(self.field)
                    self.asteroids.append(asteroid)
 
 
    def setMouseBtn(self, btn, value):
        self.mousebtn[btn] = value

            
    def controlCamera(self, task):
        #for ast in self.asteroids:
        #    ast.model.setHpr(ast.model.getH(),ast.model.getP()+0.02,ast.model.getR()+0.02)
            
        # figure out how much the mouse has moved (in pixels)
        if base.mouseWatcherNode.hasMouse():    
            mpos = base.mouseWatcherNode.getMouse()*globalClock.getDt()*50   
            if ( mpos[0] * base.win.getXSize() < -targeting_area[0] or mpos[0] * base.win.getXSize() > targeting_area[0] \
                 or  mpos[1] * base.win.getYSize() < -targeting_area[1] or mpos[1] *  base.win.getYSize() > targeting_area[1]):
                self.relplayer.setHpr(self.relplayer,-mpos[0],mpos[1],0)
                #self.forward = 0
            
        
        elapsed = task.time - self.last
        if (self.last == 0): 
            elapsed = 0
         
        forward = up = right = Vec3(0,0,0)
        if self.keyMap['q']:
            forward = self.relplayer.getMat().getRow3(1) * 2
        if self.keyMap['e']:
            forward = self.relplayer.getMat().getRow3(1) * -2
        if not self.keyMap['q'] and not self.keyMap['e']:
            forward = 0
        if self.keyMap['w']:
            up = self.relplayer.getMat().getRow3(2) * 1
        if self.keyMap['s']:
            up = self.relplayer.getMat().getRow3(2) * -1
        if self.keyMap['a']:
            right = self.relplayer.getMat().getRow3(0) * -1
        if self.keyMap['d']:
            right = self.relplayer.getMat().getRow3(0) * 1          
        
        if (self.keyMap['q'] or self.keyMap['e'] or self.keyMap['w'] or self.keyMap['s'] or self.keyMap['a'] or self.keyMap['d']):
            self.vector = (self.vector + forward + up + right)    
        self.player.setPos(self.player.getPos() +  self.vector*elapsed)

        
        tmp = self.relplayer.getHpr()

        # self.inst1.setText('vector'+ str(self.vector))
        # self.inst2.setText('Player new'+str(self.player.getPos()))
        # self.inst3.setText('camera '+str(base.camera.getRelativeVector(base.render, Vec3(0,1,0))))
        # self.inst4.setText('RelPlayer vector'+str(self.relplayer.getRelativeVector(base.render, Vec3(0,1,0))))
        # self.inst5.setText('Accel '+str(self.forward))
        #print self.drive, forward, dir    
        #self.title1.setText(' Vector ' + str(dir*self.drive))

        self.last = task.time
        self.position = self.player.getPos()
        return Task.cont
print 'Start'
a= Main()
print 'Flatten'
a.field.flattenStrong()
render.analyze()
print 'World created'
run()
